import { PrismaClient } from '@prisma/client';
import { Author } from '../types/auth';

const prisma = new PrismaClient();

export const authorService = {
  getAuthors: async (): Promise<Author[]> => {
    try {
      // 1. Получаем авторов
      const authors = await prisma.author.findMany({
        select: {
          id: true,
          displayName: true,
          avatarUrl: true,
          bio: true,
          order: true,
          okUrl: true,
          vkUrl: true,
          websiteUrl: true,
          email: true,
          userId: true,
          isActive: true,
          _count: {
            select: {articles: true}, // Подсчёт количества статей автора
          },
          user: {
            select: {
              displayName: true,
            },
          }
        },
        where: {order: {not: 0}},
        orderBy: {
          order: 'asc',
        },
      });

      // 2. Считаем сумму лайков по авторам
      const likesPerAuthor = await prisma.article.groupBy({
        by: ['authorId'],
        _sum: {
          likes: true,
        },
      });

      // 3. Строим map для быстрого доступа
      const likesMap = new Map(
          likesPerAuthor.map((item) => [item.authorId, item._sum.likes ?? 0])
      );

      return authors.map(author => ({
        id: author.id,
        displayName: author.displayName,
        avatarUrl: author.avatarUrl,
        bio: author.bio,
        order: author.order,
        okUrl: author.okUrl,
        vkUrl: author.vkUrl,
        websiteUrl: author.websiteUrl,
        email: author.email,
        articlesCount: author._count.articles, // Количество статей
        userId: author.userId,
        userDisplayName: author.user?.displayName || null,
        isActive: author.isActive,
        totalLikes: likesMap.get(author.id) ?? 0,
      }));
    } catch (error) {
      console.error('Ошибка получения авторов:', error);
      throw new Error('Ошибка получения авторов');
    }
  }
}
